@@ -19,7 +19,6 @@
 #include "memory.h"
 #include "message.h"
 #include "w32osdll/w32osdll.h"
-#include "misc/fileio.h"
 #include "misc/peimage.h"
 
 
@@ -349,7 +348,7 @@ ia32_module_load_library (Ia32Process *process,
 
  /* ja:ファイル読み込み */
   module->name = g_path_get_basename (file);
-  module->file = fileio_get_full_path (file);
+  module->file = g_path_get_absolute (file, NULL);
   image = peimage_file_load (module->file);
   if (!image)
     {
