@@ -1,6 +1,6 @@
 /*
     IA32
-    copyright (c) 1998-2010 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,21 +18,7 @@
 #include "icmui.h"
 
 
-/* SetupColorMatchingA ordinal = 1, argument = 4 */
-static gboolean icmui_SetupColorMatchingA (Ia32Thread *thread)
-{
-  return FALSE;
-}
-
-
-/* SetupColorMatchingW ordinal = 2, argument = 4 */
-static gboolean icmui_SetupColorMatchingW (Ia32Thread *thread)
-{
-  return FALSE;
-}
-
-
 Ia32Export ia32_icmui_exports[] = {
-{ 4,    1, "SetupColorMatchingA", icmui_SetupColorMatchingA},
-{ 4,    2, "SetupColorMatchingW", icmui_SetupColorMatchingW},
-{ 0,    0, NULL,                  NULL}};
+{4, 1, "SetupColorMatchingA", NULL},
+{4, 2, "SetupColorMatchingW", NULL},
+{G_MAXINT, 0, NULL, NULL}};
