@@ -1,6 +1,6 @@
 /*
     IA32
-    copyright (c) 1998-2010 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,4 +19,4 @@
 
 
 Ia32Export ia32_mfcd42d_exports[] = {
-{ 0,    0, NULL,NULL}};
+{G_MAXINT, 0, NULL, NULL}};
