@@ -1,6 +1,6 @@
 /*
     peimage
-    copyright (c) 1998-2013 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2017 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -644,6 +644,8 @@ peimage_file_enum_import (const guint8 *image)
 }
 
 
+
+
 static gint
 peimage_file_enum_resource_compare (gconstpointer a,
                                     gconstpointer b)
@@ -958,3 +960,627 @@ peimage_file_image_from_binary (const guint8  *data,
     }
   return image;
 }
+
+
+/******************************************************************************
+*                                                                             *
+* ja:モジュール関数群                                                         *
+*                                                                             *
+******************************************************************************/
+typedef struct _PeModule
+{
+  guint8 *image;
+  gchar *file, *name;
+  gint counter;
+  GList *library;
+} PeModule;
+
+
+# ifdef USE_THREAD
+G_LOCK_DEFINE_STATIC (critical);
+static volatile gboolean critical = FALSE;
+# endif /* USE_THREAD */
+
+
+static gchar *
+peimage_get_basename (const gchar *file)
+{
+  gchar *name = NULL;
+
+  if (file)
+    {
+      name = g_path_get_basename (file);
+      if (!g_strrchr (name, '.'))
+        {
+          gchar *tmp;
+
+          tmp = g_strconcat (name, ".dll", NULL);
+          g_free (name);
+          name = tmp;
+        }
+    }
+  return name;
+}
+
+
+/*  ja:既にロードされているPEイメージを取得する
+    process,プロセス
+       file,イメージ名
+        RET,PEイメージ,NULL:ロードされていない                              */
+guint8 *
+peimage_module_get_library (PeProcess   *process,
+                            const gchar *file)
+{
+  guint8 *image = NULL;
+
+  if (file)
+    {
+      gchar *base, *name;
+      GList *glist;
+
+      base = g_path_get_basename (file);
+      name = peimage_get_basename (file);
+#ifdef USE_THREAD
+      G_LOCK (critical);
+      critical = TRUE;
+#endif /* USE_THREAD */
+      for (glist = g_list_first (process->module);
+                                            glist; glist = g_list_next (glist))
+        if (g_ascii_strcasecmp (((PeModule *)glist->data)->name, base) == 0
+         || g_ascii_strcasecmp (((PeModule *)glist->data)->name, name) == 0)
+          {
+            image = ((PeModule *)glist->data)->image;
+            break;
+          }
+#ifdef USE_THREAD
+      critical = FALSE;
+      G_UNLOCK (critical);
+#endif /* USE_THREAD */
+      g_free (base);
+      g_free (name);
+    }
+  return image;
+}
+
+
+/*  ja:モジュール構造体を取得する
+    process,プロセス
+      image,PEイメージ
+        RET,モジュール構造体,NULL:エラー                                    */
+static PeModule *
+peimage_module_get_info (PeProcess    *process,
+                         const guint8 *image)
+{
+  PeModule *module = NULL;
+
+  if (image)
+    {
+      GList *glist;
+
+#ifdef USE_THREAD
+      G_LOCK (critical);
+      critical = TRUE;
+#endif /* USE_THREAD */
+      for (glist = g_list_first (process->module);
+                                            glist; glist = g_list_next (glist))
+        if (((PeModule *)glist->data)->image == image)
+          {
+            module = glist->data;
+            break;
+          }
+#ifdef USE_THREAD
+      critical = FALSE;
+      G_UNLOCK (critical);
+#endif /* USE_THREAD */
+    }
+  return module;
+}
+
+
+/*  ja:イメージをロードする
+    process,プロセス
+       file,イメージ名
+        RET,イメージ,NULL:エラー                                            */
+guint8 *
+peimage_module_load_library (PeProcess        *process,
+                             const gchar      *file,
+                             PeModuleSystem    pemodule_system,
+                             PeModuleRelocate  pemodule_relocate,
+                             PeModuleImport    pemodule_import,
+                             PeModuleEntry     pemodule_entry,
+                             gpointer          user_data)
+{
+  gboolean result = TRUE;
+  guint8 *image;
+  gint i;
+  PeModule *module;
+
+  if (!process || !file)
+    return NULL;
+  image = peimage_module_get_library (process, file);
+  if (image)
+    {
+      /* ja:既読 */
+      module = peimage_module_get_info (process, image);
+      if (module)
+        {
+          module->counter++;
+          return module->image;
+        }
+    }
+  module = g_malloc0 (sizeof (PeModule));
+  module->counter = 1;
+  module->name = peimage_get_basename (file);
+  if (pemodule_system)
+    module->image = pemodule_system (process, module->name, user_data);
+  if (module->image)
+    {
+      module->file = g_build_filename
+            (peimage_dir_get_path (PEIMAGE_DIR_SYSTEM), module->name, NULL);
+#ifdef USE_THREAD
+      G_LOCK (critical);
+      critical = TRUE;
+#endif /* USE_THREAD */
+      process->module = g_list_append (process->module, module);
+#ifdef USE_THREAD
+      critical = FALSE;
+      G_UNLOCK (critical);
+#endif /* USE_THREAD */
+      return module->image;
+    }
+  g_free (module->name);
+
+  /* ja:ファイル読み込み */
+  module->name = g_path_get_basename (file);
+  module->file = g_path_get_absolute (file, NULL);
+  module->image = peimage_file_load (module->file);
+  for (i = 0; i < 8; i++)
+    if (!module->image)
+      {
+        gchar *path;
+
+        g_free (module->name);
+        g_free (module->file);
+        module->name = i % 2 == 0 ? g_path_get_basename (file)
+                                  : peimage_get_basename (file);
+        path = i < 4 ? g_build_filename (peimage_dir_get_path
+                (i / 2 == 0 ? PEIMAGE_DIR_SYSTEM : PEIMAGE_DIR_WINDOWS),
+                                                        module->name, NULL)
+                : g_build_filename (peimage_dir_get_path (PEIMAGE_DIR_HOME),
+                                    i < 6 ? "bin" : "lib", module->name, NULL);
+        module->file = peimage_dir_get_case_insensitive (path);
+        g_free (path);
+        module->image = peimage_file_load (module->file);
+      }
+  if (!module->image)
+    {
+      g_free (module->name);
+      g_free (module->file);
+      g_free (module);
+      return NULL;
+    }
+
+  /* ja:インポート */
+  if (pe_ioh_get_data_directory_size (module->image,
+                                            PEIMAGE_DIRECTORY_ENTRY_IMPORT))
+    {
+      guint16 sections;
+      guint32 address, size;
+
+      sections = pe_ifh_get_number_of_sections (module->image);
+      size = pe_ioh_get_size_of_image (module->image);
+      for (address = pe_ioh_get_data_directory_virtual_address (module->image,
+                                            PEIMAGE_DIRECTORY_ENTRY_IMPORT);
+                            address + sizeof (ImageImportDescriptor) <= size;
+                                    address += sizeof (ImageImportDescriptor))
+        {
+          gchar *name;
+          ImageImportDescriptor *iid;
+
+          iid = (ImageImportDescriptor *)(module->image + address);
+          for (i = iid_get_name (iid); 0 < i && i < size; i++)
+            if (module->image[i] == '\0')
+              break;
+          if (i <= 0 || size <= i)
+            break;
+          name = (gchar *)(module->image + iid_get_name (iid));
+          image = peimage_module_load_library (process,
+                                               name,
+                                               pemodule_system,
+                                               pemodule_relocate,
+                                               pemodule_import,
+                                               pemodule_entry,
+                                               user_data);
+          if (image)
+            {
+              guint32 lookup, thunk;
+              ImageSectionHeader *ish;
+
+              ish = pe_image_section_header (module->image);
+              for (i = 0; i < sections; i++)
+                {
+                  if (ish_get_virtual_address (ish)
+                                        <= iid_get_original_first_thunk (iid)
+                    && iid_get_original_first_thunk (iid)
+                                                < ish_get_virtual_address (ish)
+                                                + ish_get_virtual_size (ish))
+                    break;
+                  ish++;
+                }
+              lookup = i < sections ? iid_get_original_first_thunk (iid)
+                                    : iid_get_first_thunk (iid);
+              thunk = iid_get_first_thunk (iid);
+              while (lookup + sizeof (guint32) <= size
+                                        && thunk + sizeof (guint32) <= size)
+                {
+                  gconstpointer func = NULL;
+                  ImageThunkData *ilt, *iat;
+
+                  ilt = (ImageThunkData *)(module->image + lookup);
+                  iat = (ImageThunkData *)(module->image + thunk);
+                  if (!itd_get_address_of_data (ilt)
+                                            || !itd_get_address_of_data (iat))
+                    break;
+                  if (itd_get_ordinal (ilt) & PEIMAGE_ORDINAL_FLAG)
+                    {
+                      guint16 ordinal;
+
+                      ordinal = itd_get_ordinal (ilt) & ~PEIMAGE_ORDINAL_FLAG;
+                      func = peimage_file_get_proc (image, ordinal);
+                    }
+                  else if (itd_get_address_of_data (ilt)
+                                                    + sizeof (guint16) <= size)
+                    {
+                      for (i = itd_get_address_of_data (ilt)
+                                + sizeof (guint16); 0 <= i && i < size; i++)
+                        if (module->image[i] == '\0')
+                          {
+                            const ImageImportByName *iibn;
+
+                            iibn = (const ImageImportByName *)(module->image
+                                            + itd_get_address_of_data (ilt));
+                            func = peimage_file_get_func (image,
+                                                        iibn_get_name (iibn));
+                            break;
+                          }
+                    }
+                  itd_set_function (iat, pemodule_import
+                                ? pemodule_import (process, func, user_data)
+                                : GPOINTER_TO_UINT (func));
+                  lookup += sizeof (guint32);
+                  thunk += sizeof (guint32);
+                }
+              module->library = g_list_append (module->library, image);
+            }
+          else
+            {
+              result = FALSE;
+            }
+        }
+    }
+
+  if (!result)
+    {
+      for (module->library = g_list_first (module->library); module->library;
+                                        module->library = g_list_delete_link
+                                            (module->library, module->library))
+        peimage_module_free_library (process, module->library->data,
+                                                    pemodule_entry, user_data);
+      g_free (module->image);
+      g_free (module->name);
+      g_free (module->file);
+      g_free (module);
+      return NULL;
+    }
+
+  /* ja:再配置 */
+  if (pemodule_relocate)
+    peimage_file_relocate (module->image,
+                        pemodule_relocate (process, module->image, user_data));
+
+  /* ja:プロセス登録 */
+#ifdef USE_THREAD
+  G_LOCK (critical);
+  critical = TRUE;
+#endif /* USE_THREAD */
+  process->module = g_list_append (process->module, module);
+#ifdef USE_THREAD
+  critical = FALSE;
+  G_UNLOCK (critical);
+#endif /* USE_THREAD */
+
+  if (pemodule_entry
+                && !pemodule_entry (process, module->image, TRUE, user_data))
+    {
+      peimage_module_free_library (process,
+                                    module->image, pemodule_entry, user_data);
+      return NULL;
+    }
+  return module->image;
+}
+
+
+/*  ja:イメージを解放する
+    process,プロセス
+      image,イメージ
+        RET,TRUE:正常終了,FALSE:エラー                                      */
+gboolean
+peimage_module_free_library (PeProcess     *process,
+                             const guint8  *image,
+                             PeModuleEntry  pemodule_entry,
+                             gpointer       user_data)
+{
+  gboolean result = FALSE;
+  PeModule *module;
+
+  module = peimage_module_get_info (process, image);
+  if (module)
+    {
+      result = TRUE;
+      module->counter--;
+      if (module->counter <= 0)
+        {
+          if (pemodule_entry)
+            result = pemodule_entry (process, module->image, FALSE, user_data);
+#ifdef USE_THREAD
+          G_LOCK (critical);
+          critical = TRUE;
+#endif /* USE_THREAD */
+          process->module = g_list_remove (process->module, module);
+#ifdef USE_THREAD
+          critical = FALSE;
+          G_UNLOCK (critical);
+#endif /* USE_THREAD */
+          for (module->library = g_list_first (module->library);
+                                                            module->library;
+                                        module->library = g_list_delete_link
+                                            (module->library, module->library))
+            if (!peimage_module_free_library (process, module->library->data,
+                                                    pemodule_entry, user_data))
+              result = FALSE;
+          g_free (module->image);
+          g_free (module->name);
+          g_free (module->file);
+          g_free (module);
+        }
+    }
+  return result;
+}
+
+
+/*  ja:イメージの名前を取得する
+    process,プロセス
+      image,イメージ
+        RET,名前,NULL:エラー                                                */
+const gchar *
+peimage_module_get_filename (PeProcess    *process,
+                             const guint8 *image)
+{
+  PeModule *module;
+
+  module = peimage_module_get_info (process, image);
+  return module ? module->file : NULL;
+}
+
+
+/*  ja:プロセスのイメージ一覧を取得する
+    process,プロセス
+        RET,リスト,NULL:エラー                                              */
+GList *
+peimage_module_get_list (PeProcess *process)
+{
+  GList *module = NULL, *glist;
+
+#ifdef USE_THREAD
+  G_LOCK (critical);
+  critical = TRUE;
+#endif /* USE_THREAD */
+  for (glist = g_list_first (process->module);
+                                            glist; glist = g_list_next (glist))
+    module = g_list_append (module, ((PeModule *)glist->data)->image);
+#ifdef USE_THREAD
+  critical = FALSE;
+  G_UNLOCK (critical);
+#endif /* USE_THREAD */
+  return module;
+}
+
+
+/******************************************************************************
+*                                                                             *
+* ja:ディレクトリ関数群                                                       *
+*                                                                             *
+******************************************************************************/
+/*  ja:ディレクトリを取得する
+    dir,ディレクトリの種類
+    RET,ディレクトリ                                                        */
+const gchar *
+peimage_dir_get_path (const guint dir)
+{
+  gchar *file;
+  const gchar *key, *path = NULL;
+  const static gchar *dir_windows = NULL, *dir_system = NULL, *dir_temp = NULL;
+  const static gchar *dir_data = NULL, *dir_home = NULL;
+  gint i;
+  GKeyFile *key_file;
+
+  switch (dir)
+    {
+      case PEIMAGE_DIR_WINDOWS: if (dir_windows) return dir_windows; break;
+      case PEIMAGE_DIR_SYSTEM:  if (dir_system)  return dir_system;  break;
+      case PEIMAGE_DIR_TEMP:    if (dir_temp)    return dir_temp;    break;
+      case PEIMAGE_DIR_DATA:    if (dir_data)    return dir_data;    break;
+      case PEIMAGE_DIR_HOME:    if (dir_home)    return dir_home;
+    }
+  for (i = 0; i < 2 && !path; i++)
+    {
+      switch (dir)
+        {
+          case PEIMAGE_DIR_WINDOWS:
+            file = peimage_dir_get_filename ("system.ini",
+                                i == 0 ? PEIMAGE_DIR_HOME : PEIMAGE_DIR_DATA);
+            key = "windows";
+            break;
+          case PEIMAGE_DIR_SYSTEM:
+            file = peimage_dir_get_filename ("system.ini",
+                                i == 0 ? PEIMAGE_DIR_HOME : PEIMAGE_DIR_DATA);
+            key = "system";
+            break;
+          case PEIMAGE_DIR_TEMP:
+            file = peimage_dir_get_filename ("system.ini",
+                                i == 0 ? PEIMAGE_DIR_HOME : PEIMAGE_DIR_DATA);
+            key = "temp";
+            break;
+          case PEIMAGE_DIR_DATA:
+            if (i == 0)
+              {
+                file = g_build_filename (g_get_home_dir (),
+                                    ".maid.org", "w32ldr", "system.ini", NULL);
+                key = "data";
+                break;
+              }
+          default:
+            file = NULL;
+            key = NULL;
+        }
+      if (file && key)
+        {
+          gchar *tmp, *utf8str;
+
+          key_file = g_key_file_new ();
+          tmp = peimage_dir_get_case_insensitive (file);
+          g_key_file_load_from_file (key_file, tmp, G_KEY_FILE_NONE, NULL);
+          g_free (tmp);
+          utf8str = g_key_file_get_string (key_file, "directory", key, NULL);
+          g_key_file_free (key_file);
+          if (utf8str)
+            {
+              path = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
+              g_free (utf8str);
+            }
+        }
+      g_free (file);
+    }
+  switch (dir)
+    {
+      case PEIMAGE_DIR_WINDOWS:
+        return dir_windows = path ? path
+#ifdef BINDIR
+                                  : g_build_filename (BINDIR, "w32ldr", NULL);
+#else /* not BINDIR */
+                                  : "/usr/local/bin/w32ldr";
+#endif /* not BINDIR */
+      case PEIMAGE_DIR_SYSTEM:
+        return dir_system = path ? path
+#ifdef LIBDIR
+                                 : g_build_filename (LIBDIR, "w32ldr", NULL);
+#else /* not LIBDIR */
+                                 : "/usr/local/lib/w32ldr";
+#endif /* not LIBDIR */
+      case PEIMAGE_DIR_TEMP:
+        return dir_temp = path ? path : g_get_tmp_dir ();
+      case PEIMAGE_DIR_DATA:
+        return dir_data = path ? path
+#ifdef SYSCONFDIR
+                               : g_build_filename (SYSCONFDIR, "w32ldr", NULL);
+#else /* not SYSCONFDIR */
+                               : "/usr/local/etc/w32ldr";
+#endif /* not SYSCONFDIR */
+      case PEIMAGE_DIR_HOME:
+        if (!dir_home)
+          dir_home = g_build_filename (g_get_home_dir (),
+                                                ".maid.org", "w32ldr", NULL);
+        return dir_home;
+    }
+  return NULL;
+}
+
+
+/*  ja:大文字小文字を区別しないパス名を求める
+    path,ファイル名
+     RET,大文字小文字を区別しないパス名                                     */
+gchar *
+peimage_dir_get_case_insensitive (const gchar *path)
+{
+  gchar **element, *file = NULL;
+  gint i;
+
+  if (!path)
+    return NULL;
+  if (path[0] == '\0')
+    return g_malloc0 (sizeof (gchar));
+  element = g_strsplit (path, G_DIR_SEPARATOR_S, 0);
+  for (i = 0; element[i]; i++)
+    if ((element[i])[0] == '\0')
+      {
+        if (!file)
+          file = g_strdup (G_DIR_SEPARATOR_S);
+      }
+    else
+      {
+        gchar *tmp;
+
+        tmp = file ? g_build_filename (file, element[i], NULL)
+                   : g_strdup (element[i]);
+        if (g_file_test (tmp, G_FILE_TEST_EXISTS))
+          {
+            g_free (file);
+            file = tmp;
+          }
+        else
+          {
+            const gchar *name;
+            GDir *dir;
+
+            g_free (tmp);
+            tmp = NULL;
+            dir = g_dir_open (file ? file : ".", 0, NULL);
+            while ((name = g_dir_read_name (dir)))
+              if (g_ascii_strcasecmp (element[i], name) == 0)
+                {
+                  tmp = file ? g_build_filename (file, name, NULL)
+                             : g_strdup (name);
+                  break;
+                }
+            g_dir_close (dir);
+            if (!tmp)
+              {
+                while (element[i])
+                  {
+                    if ((element[i])[0] != '\0')
+                      {
+                        tmp = file ? g_build_filename (file, element[i], NULL)
+                                   : g_strdup (element[i]);
+                        g_free (file);
+                        file = tmp;
+                      }
+                    i++;
+                  }
+                break;
+              }
+            g_free (file);
+            file = tmp;
+          }
+      }
+  g_strfreev (element);
+  return file;
+}
+
+
+/*  ja:特定のディレクトリのファイルを取得する
+    file,元のファイル名
+     dir,ディレクトリの種類
+     RET,ファイル                                                           */
+gchar *
+peimage_dir_get_filename (const gchar *file,
+                          const guint  dir)
+{
+  gchar *name, *tmp, *result;
+
+  name = g_path_get_basename (file);
+  tmp = g_build_filename (peimage_dir_get_path (dir), name, NULL);
+  g_free (name);
+  result = peimage_dir_get_case_insensitive (tmp);
+  g_free (tmp);
+  return result;
+}
