@@ -1,6 +1,6 @@
 /*
     peimage
-    copyright (c) 1998-2013 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2017 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -859,6 +859,113 @@ peimage_file_image_from_binary (const guint8  *data,
                                 const guint32  align);
 
 
+/******************************************************************************
+*                                                                             *
+* ja:モジュール関数群                                                         *
+*                                                                             *
+******************************************************************************/
+typedef struct _PeProcess
+{
+  GList *module;
+} PeProcess;
+typedef guint8 *(*PeModuleSystem)(PeProcess *, const gchar *, gpointer);
+typedef gint (*PeModuleRelocate)(PeProcess *, const guint8 *, gpointer);
+typedef guint32 (*PeModuleImport)(PeProcess *, gconstpointer, gpointer);
+typedef gboolean (*PeModuleEntry)(PeProcess *, const guint8 *, const gboolean, gpointer);
+
+
+/*  ja:既にロードされているPEイメージを取得する
+    process,プロセス
+       file,イメージ名
+        RET,PEイメージ,NULL:ロードされていない                              */
+guint8 *
+peimage_module_get_library (PeProcess   *process,
+                            const gchar *file);
+
+
+/*  ja:PEイメージをロードする
+              process,プロセス
+                 file,イメージ名
+      pemodule_system,システムイメージ用コールバック
+    pemodule_relocale,再配置用コールバック
+      pemodule_import,インポート用コールバック
+       pemodule_entry,エントリーポイント用コールバック
+            user_data,ユーザデータ
+                  RET,PEイメージ,NULL:エラー                                */
+guint8 *
+peimage_module_load_library (PeProcess        *process,
+                             const gchar      *file,
+                             PeModuleSystem    pemodule_system,
+                             PeModuleRelocate  pemodule_relocale,
+                             PeModuleImport    pemodule_import,
+                             PeModuleEntry     pemodule_entry,
+                             gpointer          user_data);
+
+
+/*  ja:PEイメージを解放する
+           process,プロセス
+             image,PEイメージ
+    pemodule_entry,エントリーポイント用コールバック
+         user_data,ユーザデータ
+               RET,TRUE:正常終了,FALSE:エラー                               */
+gboolean
+peimage_module_free_library (PeProcess     *process,
+                             const guint8  *image,
+                             PeModuleEntry  pemodule_entry,
+                             gpointer       user_data);
+
+
+/*  ja:イメージの名前を取得する
+    process,プロセス
+      image,イメージ
+        RET,名前,NULL:エラー                                                */
+const gchar *
+peimage_module_get_filename (PeProcess    *process,
+                             const guint8 *image);
+
+
+/*  ja:プロセスのイメージ一覧を取得する
+    process,プロセス
+        RET,リスト,NULL:エラー                                              */
+GList *
+peimage_module_get_list (PeProcess *process);
+
+
+/******************************************************************************
+*                                                                             *
+* ja:ディレクトリ関数群                                                       *
+*                                                                             *
+******************************************************************************/
+#define PEIMAGE_DIR_WINDOWS 0
+#define PEIMAGE_DIR_SYSTEM  1
+#define PEIMAGE_DIR_TEMP    2
+#define PEIMAGE_DIR_DATA    3
+#define PEIMAGE_DIR_HOME    4
+
+
+/*  ja:ディレクトリを取得する
+    dir,ディレクトリの種類
+    RET,ディレクトリ                                                        */
+const gchar *
+peimage_dir_get_path (const guint dir);
+
+
+/*  ja:大文字小文字を区別しないパス名を求める
+    path,ファイル名
+     RET,大文字小文字を区別しないパス名                                     */
+gchar *
+peimage_dir_get_case_insensitive (const gchar *path);
+
+
+/*  ja:特定のディレクトリのファイルを取得する
+    file,元のファイル名
+     dir,ディレクトリの種類
+     RET,ファイル                                                           */
+gchar *
+peimage_dir_get_filename (const gchar *file,
+                          const guint  dir);
+
+
 G_END_DECLS
 
 
