@@ -1,6 +1,6 @@
 /*
     common
-    copyright (c) 1998-2017 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2018 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,55 @@
 #include "common.h"
 
 
+/******************************************************************************
+*                                                                             *
+******************************************************************************/
+/*  文字が数字であるかどうかを判断する
+     ch,文字
+    RET,TRUE:数字,FALSE:その他                                              */
+BOOL WINAPI
+IsCharNumericW (WCHAR ch)
+{
+  return IsCharAlphaNumericW (ch) && !IsCharAlphaW (ch);
+}
+
+
+/*  文字が数字であるかどうかを判断する
+     ch,文字
+    RET,TRUE:数字,FALSE:その他                                              */
+BOOL WINAPI
+IsCharNumericA (CHAR ch)
+{
+  return IsCharAlphaNumericA (ch) && !IsCharAlphaA (ch);
+}
+
+
+/*  文字がコントロールコードであるかどうかを判断する
+     ch,文字
+    RET,TRUE:コントロールコード,FALSE:その他                                */
+BOOL WINAPI
+IsCharControlW (WCHAR ch)
+{
+  WORD ct;
+
+  return !GetStringTypeExW (LOCALE_USER_DEFAULT, CT_CTYPE1, &ch, 1, &ct)
+                                                || (ct & C1_CNTRL) || ct == 0;
+}
+
+
+/*  文字がコントロールコードであるかどうかを判断する
+     ch,文字
+    RET,TRUE:コントロールコード,FALSE:その他                                */
+BOOL WINAPI
+IsCharControlA (CHAR ch)
+{
+  WORD ct;
+
+  return !GetStringTypeExA (LOCALE_USER_DEFAULT, CT_CTYPE1, &ch, 1, &ct)
+                                                || (ct & C1_CNTRL) || ct == 0;
+}
+
+
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
@@ -31,12 +80,13 @@
 DWORD WINAPI
 DecW (LPCWSTR lpszText)
 {
-  DWORD dwResult = 0;
+  DWORD dwResult;
 
   if (lpszText)
     {
       LPCWSTR p;
 
+      dwResult = 0;
       for (p = lpszText; *p != '\0'; p++)
         {
           int i;
@@ -53,9 +103,16 @@ DecW (LPCWSTR lpszText)
                 break;
               }
           if (i >= n_elements (cDec))
-            return (DWORD)-1;
+            {
+              dwResult = (DWORD)-1;
+              break;
+            }
         }
     }
+  else
+    {
+      dwResult = (DWORD)-1;
+    }
   return dwResult;
 }
 
@@ -66,12 +123,13 @@ DecW (LPCWSTR lpszText)
 DWORD WINAPI
 DecA (LPCSTR lpszText)
 {
-  DWORD dwResult = 0;
+  DWORD dwResult;
 
   if (lpszText)
     {
       LPCSTR p;
 
+      dwResult = 0;
       for (p = lpszText; *p != '\0'; p++)
         {
           int i;
@@ -88,9 +146,16 @@ DecA (LPCSTR lpszText)
                 break;
               }
           if (i >= n_elements (cDec))
-            return (DWORD)-1;
+            {
+              dwResult = (DWORD)-1;
+              break;
+            }
         }
     }
+  else
+    {
+      dwResult = (DWORD)-1;
+    }
   return dwResult;
 }
 
@@ -101,12 +166,13 @@ DecA (LPCSTR lpszText)
 DWORD WINAPI
 HexW (LPCWSTR lpszText)
 {
-  DWORD dwResult = 0;
+  DWORD dwResult;
 
   if (lpszText)
     {
       LPCWSTR p;
 
+      dwResult = 0;
       for (p = lpszText; *p != '\0'; p++)
         {
           int i;
@@ -123,9 +189,16 @@ HexW (LPCWSTR lpszText)
                 break;
               }
           if (i >= n_elements (cHex))
-            return (DWORD)-1;
+            {
+              dwResult = (DWORD)-1;
+              break;
+            }
         }
     }
+  else
+    {
+      dwResult = (DWORD)-1;
+    }
   return dwResult;
 }
 
@@ -136,12 +209,13 @@ HexW (LPCWSTR lpszText)
 DWORD WINAPI
 HexA (LPCSTR lpszText)
 {
-  DWORD dwResult = 0;
+  DWORD dwResult;
 
   if (lpszText)
     {
       LPCSTR p;
 
+      dwResult = 0;
       for (p = lpszText; *p != '\0'; p++)
         {
           int i;
@@ -158,13 +232,268 @@ HexA (LPCSTR lpszText)
                 break;
               }
           if (i >= n_elements (cHex))
-            return (DWORD)-1;
+            {
+              dwResult = (DWORD)-1;
+              break;
+            }
         }
     }
+  else
+    {
+      dwResult = (DWORD)-1;
+    }
   return dwResult;
 }
 
 
+/*  ja:数値→文字列
+        nValue,数値
+    lpszString,文字列
+        nRadix,基数
+         nWide,桁数
+       fSigned,TRUE:符号あり,FALSE:符号なし                                 */
+VOID WINAPI
+ValStrW (int    nValue,
+         LPWSTR lpszString,
+         int    nRadix,
+         int    nWide,
+         BOOL   fSigned)
+{
+  if (lpszString && 2 <= nRadix && nRadix <= 16)
+    {
+      int i = 0, j;
+      const static WCHAR cHex[] = {'0', '1', '2', '3', '4', '5', '6', '7',
+                                   '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
+
+      if (nValue == 0)
+        {
+          lpszString[i++] = cHex[0];
+        }
+      else
+        {
+          int t;
+
+          t = nValue;
+          if (fSigned && nValue < 0)
+            {
+              while (t != 0)
+                {
+                  lpszString[i++] = cHex[abs (t % nRadix)];
+                  t /= nRadix;
+                }
+            }
+          else
+            {
+              while (t != 0)
+                {
+                  lpszString[i++] = cHex[(unsigned)t % nRadix];
+                  t = (unsigned)t / nRadix;
+                }
+            }
+        }
+      if (fSigned && nValue < 0)
+        lpszString[i++] = '-';
+      if (nWide > 0)
+        {
+          while (i < nWide)
+            lpszString[i++] = ' ';
+        }
+      else if (nWide < 0)
+        {
+          for (j = i; j < -nWide; j++)
+            lpszString[j] = cHex[0];
+          if (lpszString[i - 1] == '-')
+            {
+              lpszString[i - 1] = cHex[0];
+              lpszString[j - 1] = '-';
+            }
+          i = j;
+        }
+      lpszString[i] = '\0';
+      for (j = 0; j < i / 2; j++)
+        {
+          WCHAR c;
+
+          c = lpszString[j];
+          lpszString[j] = lpszString[i - j - 1];
+          lpszString[i - j - 1] = c;
+        }
+    }
+}
+
+
+/*  ja:数値→文字列
+        nValue,数値
+    lpszString,文字列
+        nRadix,基数
+         nWide,桁数
+       fSigned,TRUE:符号あり,FALSE:符号なし                                 */
+VOID WINAPI
+ValStrA (int   nValue,
+         LPSTR lpszString,
+         int   nRadix,
+         int   nWide,
+         BOOL  fSigned)
+{
+  if (lpszString && 2 <= nRadix && nRadix <= 16)
+    {
+      int i = 0, j;
+      const static CHAR cHex[] = {'0', '1', '2', '3', '4', '5', '6', '7',
+                                  '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
+
+      if (nValue == 0)
+        {
+          lpszString[i++] = cHex[0];
+        }
+      else
+        {
+          int t;
+
+          t = nValue;
+          if (fSigned && nValue < 0)
+            {
+              while (t != 0)
+                {
+                  lpszString[i++] = cHex[abs (t % nRadix)];
+                  t /= nRadix;
+                }
+            }
+          else
+            {
+              while (t != 0)
+                {
+                  lpszString[i++] = cHex[(unsigned)t % nRadix];
+                  t = (unsigned)t / nRadix;
+                }
+            }
+        }
+      if (fSigned && nValue < 0)
+        lpszString[i++] = '-';
+      if (nWide > 0)
+        {
+          while (i < nWide)
+            lpszString[i++] = ' ';
+        }
+      else if (nWide < 0)
+        {
+          for (j = i; j < -nWide; j++)
+            lpszString[j] = cHex[0];
+          if (lpszString[i - 1] == '-')
+            {
+              lpszString[i - 1] = cHex[0];
+              lpszString[j - 1] = '-';
+            }
+          i = j;
+        }
+      lpszString[i] = '\0';
+      for (j = 0; j < i / 2; j++)
+        {
+          CHAR c;
+
+          c = lpszString[j];
+          lpszString[j] = lpszString[i - j - 1];
+          lpszString[i - j - 1] = c;
+        }
+    }
+}
+
+
+/*  ja:文字列→数値
+        nValue,数値
+    lpszString,文字列
+        nRadix,基数
+           RET,TRUE:正常終了,FALSE:エラー                                   */
+BOOL WINAPI
+StrValW (int     *nValue,
+         LPCWSTR  lpszString,
+         int      nRadix)
+{
+  BOOL fResult = FALSE;
+
+  if (nValue && lpszString && 2 <= nRadix && nRadix <= 16)
+    {
+      unsigned s = 0;
+      BOOL fSign;
+      LPCWSTR p, lpszHex = L"0123456789abcdef";
+
+      for (p = lpszString; *p != '\0' && *p != '-'; p++)
+        {
+          LPWSTR q;
+
+          q = StrChrIW (lpszHex, *p);
+          if (q && q - lpszHex < nRadix)
+            break;
+        }
+      if (fSign = (*p == '-'))
+        p++;
+      while (*p != '\0')
+        {
+          int n;
+          LPWSTR q;
+
+          s *= nRadix;
+          q = StrChrIW (lpszHex, *p);
+          if (!q || (n = q - lpszHex) >= nRadix)
+            break;
+          s += n;
+          p++;
+          fResult = TRUE;
+        }
+      if (fResult)
+        *nValue = fSign ? -s : s;
+    }
+  return fResult;
+}
+
+
+/*  ja:文字列→数値
+        nValue,数値
+    lpszString,文字列
+        nRadix,基数
+           RET,TRUE:正常終了,FALSE:エラー                                   */
+BOOL WINAPI
+StrValA (int    *nValue,
+         LPCSTR  lpszString,
+         int     nRadix)
+{
+  BOOL fResult = FALSE;
+
+  if (nValue && lpszString && 2 <= nRadix && nRadix <= 16)
+    {
+      unsigned s = 0;
+      BOOL fSign;
+      LPCSTR p, lpszHex = "0123456789abcdef";
+
+      for (p = lpszString; *p != '\0' && *p != '-'; p++)
+        {
+          LPSTR q;
+
+          q = StrChrIA (lpszHex, *p);
+          if (q && q - lpszHex < nRadix)
+            break;
+        }
+      if (fSign = (*p == '-'))
+        p++;
+      while (*p != '\0')
+        {
+          int n;
+          LPSTR q;
+
+          s *= nRadix;
+          q = StrChrIA (lpszHex, *p);
+          if (!q || (n = q - lpszHex) >= nRadix)
+            break;
+          s += n;
+          p++;
+          fResult = TRUE;
+        }
+      if (fResult)
+        *nValue = fSign ? -s : s;
+    }
+  return fResult;
+}
+
+
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
@@ -436,25 +765,19 @@ LPWSTR WINAPI
 StringDuplicateExW (LPCWSTR lpszText,
                     SSIZE_T nSize)
 {
-  LPWSTR lpszResult;
+  LPWSTR lpszResult = NULL;
 
-  if (nSize > 0)
+  if (lpszText)
     {
-      if (lpszText && lpszText[nSize - 1] != '\0')
+      if (nSize > 0 && lpszText[nSize - 1] != '\0')
         nSize++;
-    }
-  else if (lpszText && nSize < 0)
-    {
-      nSize = lstrlenW (lpszText) + 1;
-    }
-  if (nSize > 0)
-    {
-      lpszResult = MemoryAlloc (nSize * sizeof (WCHAR));
-      MemoryCopy (lpszResult, lpszText, (nSize - 1) * sizeof (WCHAR));
-    }
-  else
-    {
-      lpszResult = NULL;
+      else if (nSize < 0)
+        nSize = lstrlenW (lpszText) + 1;
+      if (nSize > 0)
+        {
+          lpszResult = MemoryAlloc (nSize * sizeof (WCHAR));
+          MemoryCopy (lpszResult, lpszText, (nSize - 1) * sizeof (WCHAR));
+        }
     }
   return lpszResult;
 }
@@ -468,25 +791,19 @@ LPSTR WINAPI
 StringDuplicateExA (LPCSTR  lpszText,
                     SSIZE_T nSize)
 {
-  LPSTR lpszResult;
+  LPSTR lpszResult = NULL;
 
-  if (nSize > 0)
+  if (lpszText)
     {
-      if (lpszText && lpszText[nSize - 1] != '\0')
+      if (nSize > 0 && lpszText[nSize - 1] != '\0')
         nSize++;
-    }
-  else if (lpszText && nSize < 0)
-    {
-      nSize = lstrlenA (lpszText) + 1;
-    }
-  if (nSize > 0)
-    {
-      lpszResult = MemoryAlloc (nSize * sizeof (CHAR));
-      MemoryCopy (lpszResult, lpszText, (nSize - 1) * sizeof (CHAR));
-    }
-  else
-    {
-      lpszResult = NULL;
+      else if (nSize < 0)
+        nSize = lstrlenA (lpszText) + 1;
+      if (nSize > 0)
+        {
+          lpszResult = MemoryAlloc (nSize * sizeof (CHAR));
+          MemoryCopy (lpszResult, lpszText, (nSize - 1) * sizeof (CHAR));
+        }
     }
   return lpszResult;
 }
@@ -748,153 +1065,9 @@ StringCanonicalizeA (LPCSTR lpszString)
 }
 
 
-/*  文字列を分割する
-    lpszString,文字列
-    cDelimiter,文字
-           RET,分割された文字列                                             */
-LPWSTR * WINAPI
-StringSplitW (LPCWSTR lpszString,
-              WCHAR   cDelimiter)
-{
-  LPWSTR *plpszArray = NULL;
-
-  if (lpszString)
-    {
-      int nCount = 2;
-      LPCWSTR p;
-      LPWSTR q, *r;
-
-      for (p = lpszString; *p != '\0'; p++)
-        if (*p == cDelimiter)
-          nCount++;
-      plpszArray = MemoryAlloc (nCount * sizeof (LPWSTR)
-                                    + (p - lpszString + 1) * sizeof (WCHAR));
-      q = (LPWSTR)(plpszArray + nCount);
-      lstrcpyW (q, lpszString);
-      r = plpszArray;
-      *r++ = q;
-      while (*q != '\0')
-        {
-          if (*q == cDelimiter)
-            {
-              *r++ = q + 1;
-              *q = '\0';
-            }
-          q++;
-        }
-    }
-  return plpszArray;
-}
-
-
-/*  文字列を分割する
-    lpszString,文字列
-    cDelimiter,文字
-           RET,分割された文字列                                             */
-LPSTR * WINAPI
-StringSplitA (LPCSTR lpszString,
-              CHAR   cDelimiter)
-{
-  LPSTR *plpszArray = NULL;
-
-  if (lpszString)
-    {
-      int nCount = 2;
-      LPCSTR p;
-      LPSTR q, *r;
-
-      for (p = lpszString; *p != '\0';
-                                    p += IsDBCSLeadByteEx (CP_ACP, *p) ? 2 : 1)
-        if (*p == cDelimiter)
-          nCount++;
-      plpszArray = MemoryAlloc (nCount * sizeof (LPSTR)
-                                    + (p - lpszString + 1) * sizeof (CHAR));
-      q = (LPSTR)(plpszArray + nCount);
-      lstrcpyA (q, lpszString);
-      r = plpszArray;
-      *r++ = q;
-      while (*q != '\0')
-        {
-          if (*q == cDelimiter)
-            {
-              *r++ = q + 1;
-              *q = '\0';
-            }
-          q += IsDBCSLeadByteEx (CP_ACP, *q) ? 2 : 1;
-        }
-    }
-  return plpszArray;
-}
-
-
-/*  文字列を結合する
-    lpszSeparator,区切り文字列
-       plpszArray,分割された文字列
-              RET,文字列                                                    */
-LPWSTR WINAPI
-StringJoinVW (LPCWSTR  lpszSeparator,
-              LPCWSTR *plpszArray)
-{
-  LPWSTR lpszResult = NULL;
-
-  if (plpszArray)
-    {
-      int i, nLen = 1;
-
-      for (i = 0; plpszArray[i]; i++)
-        nLen += lstrlenW (plpszArray[i]);
-      if (lpszSeparator && i > 1)
-        nLen += lstrlenW (lpszSeparator) * (i - 1);
-      if (nLen > 1)
-        {
-          lpszResult = MemoryAlloc (nLen * sizeof (WCHAR));
-          while (*plpszArray)
-            {
-              lstrcatW (lpszResult, *plpszArray);
-              plpszArray++;
-              if (lpszSeparator && *plpszArray)
-                lstrcatW (lpszResult, lpszSeparator);
-            }
-        }
-    }
-  return lpszResult;
-}
-
-
-/*  文字列を結合する
-    lpszSeparator,区切り文字列
-       plpszArray,分割された文字列
-              RET,文字列                                                    */
-LPSTR WINAPI
-StringJoinVA (LPCSTR  lpszSeparator,
-              LPCSTR *plpszArray)
-{
-  LPSTR lpszResult = NULL;
-
-  if (plpszArray)
-    {
-      int i, nLen = 1;
-
-      for (i = 0; plpszArray[i]; i++)
-        nLen += lstrlenA (plpszArray[i]);
-      if (lpszSeparator && i > 1)
-        nLen += lstrlenA (lpszSeparator) * (i - 1);
-      if (nLen > 1)
-        {
-          lpszResult = MemoryAlloc (nLen * sizeof (CHAR));
-          while (*plpszArray)
-            {
-              lstrcatA (lpszResult, *plpszArray);
-              plpszArray++;
-              if (lpszSeparator && *plpszArray)
-                lstrcatA (lpszResult, lpszSeparator);
-            }
-        }
-    }
-  return lpszResult;
-}
-
-
+/******************************************************************************
+*                                                                             *
+******************************************************************************/
 /*  文字列を結合する
     lpszSeparator,区切り文字列
               RET,文字列                                                    */
@@ -971,11 +1144,79 @@ StringJoinA (LPCSTR lpszSeparator, ...)
 }
 
 
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+       plpszArray,分割された文字列
+              RET,文字列                                                    */
+LPWSTR WINAPI
+StringJoinVW (LPCWSTR  lpszSeparator,
+              LPCWSTR *plpszArray)
+{
+  LPWSTR lpszResult = NULL;
+
+  if (plpszArray)
+    {
+      int i, nLen = 1;
+
+      for (i = 0; plpszArray[i]; i++)
+        nLen += lstrlenW (plpszArray[i]);
+      if (lpszSeparator && i > 1)
+        nLen += lstrlenW (lpszSeparator) * (i - 1);
+      if (nLen > 1)
+        {
+          lpszResult = MemoryAlloc (nLen * sizeof (WCHAR));
+          while (*plpszArray)
+            {
+              lstrcatW (lpszResult, *plpszArray);
+              plpszArray++;
+              if (lpszSeparator && *plpszArray)
+                lstrcatW (lpszResult, lpszSeparator);
+            }
+        }
+    }
+  return lpszResult;
+}
+
+
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+       plpszArray,分割された文字列
+              RET,文字列                                                    */
+LPSTR WINAPI
+StringJoinVA (LPCSTR  lpszSeparator,
+              LPCSTR *plpszArray)
+{
+  LPSTR lpszResult = NULL;
+
+  if (plpszArray)
+    {
+      int i, nLen = 1;
+
+      for (i = 0; plpszArray[i]; i++)
+        nLen += lstrlenA (plpszArray[i]);
+      if (lpszSeparator && i > 1)
+        nLen += lstrlenA (lpszSeparator) * (i - 1);
+      if (nLen > 1)
+        {
+          lpszResult = MemoryAlloc (nLen * sizeof (CHAR));
+          while (*plpszArray)
+            {
+              lstrcatA (lpszResult, *plpszArray);
+              plpszArray++;
+              if (lpszSeparator && *plpszArray)
+                lstrcatA (lpszResult, lpszSeparator);
+            }
+        }
+    }
+  return lpszResult;
+}
+
+
 /*  分割された文字列の数を求める
     plpszArray,分割された文字列
            RET,分割された文字列の数                                         */
 int WINAPI
-StringArrayLengthW (LPCWSTR *plpszArray)
+StringLengthVW (LPCWSTR *plpszArray)
 {
   int nResult = 0;
 
@@ -990,7 +1231,7 @@ StringArrayLengthW (LPCWSTR *plpszArray)
     plpszArray,分割された文字列
            RET,分割された文字列の数                                         */
 int WINAPI
-StringArrayLengthA (LPCSTR *plpszArray)
+StringLengthVA (LPCSTR *plpszArray)
 {
   int nResult = 0;
 
@@ -1001,6 +1242,523 @@ StringArrayLengthA (LPCSTR *plpszArray)
 }
 
 
+/*  分割された文字列をコピーする
+    plpszArray,分割された文字列
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI
+StringCopyVW (LPCWSTR *plpszArray)
+{
+  LPWSTR *plpszResult = NULL;
+
+  if (plpszArray)
+    {
+      int nCount = 0, nLength = 0;
+      LPCWSTR *pp;
+
+      for (pp = plpszArray; *pp; pp++)
+        {
+          nLength += lstrlenW (*pp) + 1;
+          nCount++;
+        }
+      if (nCount > 0)
+        {
+          LPWSTR q, *qq;
+
+          qq = plpszResult = MemoryAlloc (++nCount * sizeof (LPWSTR)
+                                                + nLength * sizeof (WCHAR));
+          q = (LPWSTR)(plpszResult + nCount);
+          for (pp = plpszArray; *pp; pp++)
+            {
+              *qq++ = q;
+              nLength = lstrlenW (*pp) + 1;
+              MemoryCopy (q, *pp, nLength * sizeof (WCHAR));
+              q += nLength;
+            }
+        }
+    }
+  return plpszResult;
+}
+
+
+/*  分割された文字列をコピーする
+    plpszArray,分割された文字列
+           RET,分割された文字列                                             */
+LPSTR * WINAPI
+StringCopyVA (LPCSTR *plpszArray)
+{
+  LPSTR *plpszResult = NULL;
+
+  if (plpszArray)
+    {
+      int nCount = 0, nLength = 0;
+      LPCSTR *pp;
+
+      for (pp = plpszArray; *pp; pp++)
+        {
+          nLength += lstrlenA (*pp) + 1;
+          nCount++;
+        }
+      if (nCount > 0)
+        {
+          LPSTR q, *qq;
+
+          qq = plpszResult = MemoryAlloc (++nCount * sizeof (LPSTR)
+                                                + nLength * sizeof (CHAR));
+          q = (LPSTR)(plpszResult + nCount);
+          for (pp = plpszArray; *pp; pp++)
+            {
+              *qq++ = q;
+              nLength = lstrlenA (*pp) + 1;
+              MemoryCopy (q, *pp, nLength * sizeof (CHAR));
+              q += nLength;
+            }
+        }
+    }
+  return plpszResult;
+}
+
+
+/*  分割された文字列のNULL文字列を削除する
+    plpszArray,分割された文字列
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI
+StringShrinkVW (LPCWSTR *plpszArray)
+{
+  LPWSTR *plpszResult = NULL;
+
+  if (plpszArray)
+    {
+      int nCount = 0, nLength = 0;
+      LPCWSTR *pp;
+
+      for (pp = plpszArray; *pp; pp++)
+        if (**pp != '\0')
+          {
+            nLength += lstrlenW (*pp) + 1;
+            nCount++;
+          }
+      if (nCount > 0)
+        {
+          LPWSTR q, *qq;
+
+          qq = plpszResult = MemoryAlloc (++nCount * sizeof (LPWSTR)
+                                                + nLength * sizeof (WCHAR));
+          q = (LPWSTR)(plpszResult + nCount);
+          for (pp = plpszArray; *pp; pp++)
+            if (**pp != '\0')
+              {
+                *qq++ = q;
+                nLength = lstrlenW (*pp) + 1;
+                MemoryCopy (q, *pp, nLength * sizeof (WCHAR));
+                q += nLength;
+              }
+        }
+    }
+  return plpszResult;
+}
+
+
+/*  分割された文字列のNULL文字列を削除する
+    plpszArray,分割された文字列
+           RET,分割された文字列                                             */
+LPSTR * WINAPI
+StringShrinkVA (LPCSTR *plpszArray)
+{
+  LPSTR *plpszResult = NULL;
+
+  if (plpszArray)
+    {
+      int nCount = 0, nLength = 0;
+      LPCSTR *pp;
+
+      for (pp = plpszArray; *pp; pp++)
+        if (**pp != '\0')
+          {
+            nLength += lstrlenA (*pp) + 1;
+            nCount++;
+          }
+      if (nCount > 0)
+        {
+          LPSTR q, *qq;
+
+          qq = plpszResult = MemoryAlloc (++nCount * sizeof (LPSTR)
+                                                + nLength * sizeof (CHAR));
+          q = (LPSTR)(plpszResult + nCount);
+          for (pp = plpszArray; *pp; pp++)
+            if (**pp != '\0')
+              {
+                *qq++ = q;
+                nLength = lstrlenA (*pp) + 1;
+                MemoryCopy (q, *pp, nLength * sizeof (CHAR));
+                q += nLength;
+              }
+        }
+    }
+  return plpszResult;
+}
+
+
+/******************************************************************************
+*                                                                             *
+******************************************************************************/
+static int CALLBACK
+StringSplitDelimiterCallbackW (LPCWSTR lpszString,
+                               LPVOID  lpData)
+{
+  return lpszString && lpData && StrChrW (lpData, *lpszString) ? 1 : 0;
+}
+
+
+static int CALLBACK
+StringSplitDelimiterCallbackA (LPCSTR lpszString,
+                               LPVOID lpData)
+{
+  return lpszString && lpData && StrChrA (lpData, *lpszString) ? 1 : 0;
+}
+
+
+static int CALLBACK
+StringSplitDelimiterCallbackIW (LPCWSTR lpszString,
+                                LPVOID  lpData)
+{
+  return lpszString && lpData && StrChrIW (lpData, *lpszString) ? 1 : 0;
+}
+
+
+static int CALLBACK
+StringSplitDelimiterCallbackIA (LPCSTR lpszString,
+                                LPVOID lpData)
+{
+  return lpszString && lpData && StrChrIA (lpData, *lpszString) ? 1 : 0;
+}
+
+
+static int CALLBACK
+StringSplitDelimitersCallbackW (LPCWSTR lpszString,
+                                LPVOID  lpData)
+{
+  return lpszString && lpData ? StrSpnW (lpszString, lpData) : 0;
+}
+
+
+static int CALLBACK
+StringSplitDelimitersCallbackA (LPCSTR lpszString,
+                                LPVOID lpData)
+{
+  return lpszString && lpData ? StrSpnA (lpszString, lpData) : 0;
+}
+
+
+static int CALLBACK
+StringSplitDelimitersCallbackIW (LPCWSTR lpszString,
+                                 LPVOID  lpData)
+{
+  if (lpszString && lpData)
+    {
+      LPCWSTR p;
+
+      for (p = lpszString; *p != '\0'; p++)
+        if (!StrChrIW (lpData, *p))
+          break;
+      return p - lpszString;
+    }
+  return 0;
+}
+
+
+static int CALLBACK
+StringSplitDelimitersCallbackIA (LPCSTR lpszString,
+                                 LPVOID lpData)
+{
+  if (lpszString && lpData)
+    {
+      LPCSTR p;
+
+      for (p = lpszString; *p != '\0'; p++)
+        if (IsDBCSLeadByteEx (CP_ACP, *p) || !StrChrIA (lpData, *p))
+          break;
+      return p - lpszString;
+    }
+  return 0;
+}
+
+
+static int CALLBACK
+StringSplitSeparatorCallbackW (LPCWSTR lpszString,
+                               LPVOID  lpData)
+{
+  if (lpszString && lpData)
+    {
+      int n;
+
+      n = lstrlenW (lpData);
+      if (StrCmpNW (lpszString, lpData, n) == 0)
+        return n;
+    }
+  return 0;
+}
+
+
+static int CALLBACK
+StringSplitSeparatorCallbackA (LPCSTR lpszString,
+                               LPVOID lpData)
+{
+  if (lpszString && lpData)
+    {
+      int n;
+
+      n = lstrlenA (lpData);
+      if (StrCmpNA (lpszString, lpData, n) == 0)
+        return n;
+    }
+  return 0;
+}
+
+
+static int CALLBACK
+StringSplitSeparatorCallbackIW (LPCWSTR lpszString,
+                                LPVOID  lpData)
+{
+  if (lpszString && lpData)
+    {
+      int n;
+
+      n = lstrlenW (lpData);
+      if (StrCmpNIW (lpszString, lpData, n) == 0)
+        return n;
+    }
+  return 0;
+}
+
+
+static int CALLBACK
+StringSplitSeparatorCallbackIA (LPCSTR lpszString,
+                                LPVOID lpData)
+{
+  if (lpszString && lpData)
+    {
+      int n;
+
+      n = lstrlenA (lpData);
+      if (StrCmpNIA (lpszString, lpData, n) == 0)
+        return n;
+    }
+  return 0;
+}
+
+
+static int CALLBACK
+StringSplitComplementCallbackW (LPCWSTR lpszString,
+                                LPVOID  lpData)
+{
+  return lpszString && lpData ? StrCSpnW (lpszString, lpData) : 0;
+}
+
+
+static int CALLBACK
+StringSplitComplementCallbackA (LPCSTR lpszString,
+                                LPVOID lpData)
+{
+  return lpszString && lpData ? StrCSpnA (lpszString, lpData) : 0;
+}
+
+
+static int CALLBACK
+StringSplitComplementCallbackIW (LPCWSTR lpszString,
+                                 LPVOID  lpData)
+{
+  return lpszString && lpData ? StrCSpnIW (lpszString, lpData) : 0;
+}
+
+
+static int CALLBACK
+StringSplitComplementCallbackIA (LPCSTR lpszString,
+                                 LPVOID lpData)
+{
+  return lpszString && lpData ? StrCSpnIA (lpszString, lpData) : 0;
+}
+
+
+/*  文字列を分割する
+         lpszString,文字列
+    StringSplitProc,コールバック関数
+             lpData,ユーザデータ
+                RET,分割された文字列                                        */
+LPWSTR * WINAPI
+StringSplitW (LPCWSTR            lpszString,
+              StringSplitProcW_t StringSplitProc,
+              LPVOID             lpData)
+{
+  LPWSTR *plpszArray = NULL;
+
+  if (lpszString && StringSplitProc)
+    {
+      int nCount = 0, nLength = 0, *lpnStatus, *q;
+      LPCWSTR p;
+      LPWSTR r, *pp;
+
+      p = lpszString;
+      q = lpnStatus = MemoryAlloc ((lstrlenW (lpszString) + 1) * sizeof (int));
+      while (*p != '\0')
+        {
+          *q = StringSplitProc (p, lpData);
+          if (*q > 0)
+            {
+              nCount++;
+              p += *q;
+              q += *q;
+            }
+          else if (*q < 0)
+            {
+              goto final;
+            }
+          else
+            {
+              nLength++;
+              p++;
+              q++;
+            }
+        }
+      pp = plpszArray = MemoryAlloc ((nCount + 2) * sizeof (LPWSTR)
+                                    + (nLength + nCount + 1) * sizeof (WCHAR));
+      p = lpszString;
+      q = lpnStatus;
+      r = (LPWSTR)(plpszArray + nCount + 2);
+      *pp++ = r;
+      while (*p != '\0')
+        {
+          if (*q > 0)
+            {
+              p += *q;
+              q += *q;
+              *pp++ = ++r;
+            }
+          else
+            {
+              *r++ = *p++;
+              q++;
+            }
+        }
+      final:
+      MemoryFree (lpnStatus);
+    }
+  return plpszArray;
+}
+
+
+/*  文字列を分割する
+         lpszString,文字列
+    StringSplitProc,コールバック関数
+             lpData,ユーザデータ
+                RET,分割された文字列                                        */
+LPSTR * WINAPI
+StringSplitA (LPCSTR             lpszString,
+              StringSplitProcA_t StringSplitProc,
+              LPVOID             lpData)
+{
+  LPSTR *plpszArray = NULL;
+
+  if (lpszString && StringSplitProc)
+    {
+      int nCount = 0, nLength = 0, *lpnStatus, *q;
+      LPCSTR p;
+      LPSTR r, *pp;
+
+      p = lpszString;
+      q = lpnStatus = MemoryAlloc ((lstrlenA (lpszString) + 1) * sizeof (int));
+      while (*p != '\0')
+        {
+          *q = StringSplitProc (p, lpData);
+          if (*q > 0)
+            {
+              nCount++;
+              p += *q;
+              q += *q;
+            }
+          else if (*q < 0)
+            {
+              goto final;
+            }
+          else if (IsDBCSLeadByteEx (CP_ACP, *p))
+            {
+              nLength += 2;
+              p += 2;
+              q += 2;
+            }
+          else
+            {
+              nLength++;
+              p++;
+              q++;
+            }
+        }
+      pp = plpszArray = MemoryAlloc ((nCount + 2) * sizeof (LPSTR)
+                                    + (nLength + nCount + 1) * sizeof (CHAR));
+      p = lpszString;
+      q = lpnStatus;
+      r = (LPSTR)(plpszArray + nCount + 2);
+      *pp++ = r;
+      while (*p != '\0')
+        {
+          if (*q > 0)
+            {
+              p += *q;
+              q += *q;
+              *pp++ = ++r;
+            }
+          else
+            {
+              *r++ = *p++;
+              q++;
+            }
+        }
+      final:
+      MemoryFree (lpnStatus);
+    }
+  return plpszArray;
+}
+
+
+#define StringSplitProcedureW(prefix,suffix) \
+LPWSTR * WINAPI \
+StringSplit ## prefix ## suffix (LPCWSTR lpszString, \
+                                 LPCWSTR lpszData) \
+{ \
+  return StringSplitW (lpszString, \
+                       StringSplit ## prefix ## Callback ## suffix, \
+                       (LPVOID)lpszData); \
+}
+
+
+#define StringSplitProcedureA(prefix,suffix) \
+LPSTR * WINAPI \
+StringSplit ## prefix ## suffix (LPCSTR lpszString, \
+                                 LPCSTR lpszData) \
+{ \
+  return StringSplitA (lpszString, \
+                       StringSplit ## prefix ## Callback ## suffix, \
+                       (LPVOID)lpszData); \
+}
+
+
+StringSplitProcedureW(Delimiter,W)
+StringSplitProcedureA(Delimiter,A)
+StringSplitProcedureW(Delimiter,IW)
+StringSplitProcedureA(Delimiter,IA)
+StringSplitProcedureW(Delimiters,W)
+StringSplitProcedureA(Delimiters,A)
+StringSplitProcedureW(Delimiters,IW)
+StringSplitProcedureA(Delimiters,IA)
+StringSplitProcedureW(Separator,W)
+StringSplitProcedureA(Separator,A)
+StringSplitProcedureW(Separator,IW)
+StringSplitProcedureA(Separator,IA)
+StringSplitProcedureW(Complement,W)
+StringSplitProcedureA(Complement,A)
+StringSplitProcedureW(Complement,IW)
+StringSplitProcedureA(Complement,IA)
+
+
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
