@@ -1,6 +1,6 @@
 /*
     common
-    copyright (c) 1998-2017 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2018 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,6 +30,46 @@ extern "C" {
 #define n_elements(arr) (sizeof (arr) / sizeof ((arr)[0]))
 
 
+/******************************************************************************
+*                                                                             *
+******************************************************************************/
+/*  文字が数字であるかどうかを判断する
+     ch,文字
+    RET,TRUE:数字,FALSE:その他                                              */
+BOOL WINAPI
+IsCharNumericW (WCHAR ch);
+
+
+/*  文字が数字であるかどうかを判断する
+     ch,文字
+    RET,TRUE:数字,FALSE:その他                                              */
+BOOL WINAPI
+IsCharNumericA (CHAR ch);
+
+
+/*  文字がコントロールコードであるかどうかを判断する
+     ch,文字
+    RET,TRUE:コントロールコード,FALSE:その他                                */
+BOOL WINAPI
+IsCharControlW (WCHAR ch);
+
+
+/*  文字がコントロールコードであるかどうかを判断する
+     ch,文字
+    RET,TRUE:コントロールコード,FALSE:その他                                */
+BOOL WINAPI
+IsCharControlA (CHAR ch);
+
+
+#ifdef UNICODE
+# define IsCharNumeric IsCharNumericW
+# define IsCharControl IsCharControlW
+#else /* not UNICODE */
+# define IsCharNumeric IsCharNumericA
+# define IsCharControl IsCharControlA
+#endif /* not UNICODE */
+
+
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
@@ -61,12 +101,66 @@ DWORD WINAPI
 HexA (LPCSTR lpszText);
 
 
+/*  ja:数値→文字列
+        nValue,数値
+    lpszString,文字列
+        nRadix,基数
+         nWide,桁数
+       fSigned,TRUE:符号あり,FALSE:符号なし                                 */
+VOID WINAPI
+ValStrW (int    nValue,
+         LPWSTR lpszString,
+         int    nRadix,
+         int    nWide,
+         BOOL   fSigned);
+
+
+/*  ja:数値→文字列
+        nValue,数値
+    lpszString,文字列
+        nRadix,基数
+         nWide,桁数
+       fSigned,TRUE:符号あり,FALSE:符号なし                                 */
+VOID WINAPI
+ValStrA (int   nValue,
+         LPSTR lpszString,
+         int   nRadix,
+         int   nWide,
+         BOOL  fSigned);
+
+
+/*  ja:文字列→数値
+        nValue,数値
+    lpszString,文字列
+        nRadix,基数
+           RET,TRUE:正常終了,FALSE:エラー                                   */
+BOOL WINAPI
+StrValW (int     *nValue,
+         LPCWSTR  lpszString,
+         int      nRadix);
+
+
+/*  ja:文字列→数値
+        nValue,数値
+    lpszString,文字列
+        nRadix,基数
+           RET,TRUE:正常終了,FALSE:エラー                                   */
+BOOL WINAPI
+StrValA (int    *nValue,
+         LPCSTR  lpszString,
+         int     nRadix);
+
+
 #ifdef UNICODE
 # define Dec DecW
 # define Hex HexW
+# define ValStr ValStrW
+# define StrVal StrValW
 #else /* not UNICODE */
 # define Dec DecA
 # define Hex HexA
+# define ValStr ValStrA
+# define StrVal StrValA
 #endif /* not UNICODE */
 
 
@@ -287,22 +381,54 @@ LPSTR WINAPI
 StringCanonicalizeA (LPCSTR lpszString);
 
 
-/*  文字列を分割する
-    lpszString,文字列
-    cDelimiter,文字
-           RET,分割された文字列                                             */
-LPWSTR * WINAPI
-StringSplitW (LPCWSTR lpszString,
-              WCHAR   cDelimiter);
+#ifdef UNICODE
+# define wvasprintf wvasprintfW
+# define wasprintf wasprintfW
+# define StringDuplicateEx StringDuplicateExW
+# define StringDuplicate StringDuplicateW
+# define StringFromMultiByteEx StringMultiByteToWideCharEx
+# define StringFromMultiByte StringMultiByteToWideChar
+# define StringFromWideCharEx StringDuplicateExW
+# define StringFromWideChar StringDuplicateW
+# define StringToMultiByteEx StringWideCharToMultiByteEx
+# define StringToMultiByte StringWideCharToMultiByte
+# define StringToWideCharEx StringDuplicateExW
+# define StringToWideChar StringDuplicateW
+# define StringCanonicalizeEx StringCanonicalizeExW
+# define StringCanonicalize StringCanonicalizeW
+#else /* not UNICODE */
+# define wvasprintf wvasprintfA
+# define wasprintf wasprintfA
+# define StringDuplicateEx StringDuplicateExA
+# define StringDuplicate StringDuplicateA
+# define StringFromMultiByteEx StringDuplicateExA
+# define StringFromMultiByte StringDuplicateA
+# define StringFromWideCharEx StringWideCharToMultiByteEx
+# define StringFromWideChar StringWideCharToMultiByte
+# define StringToMultiByteEx StringDuplicateExA
+# define StringToMultiByte StringDuplicateA
+# define StringToWideCharEx StringMultiByteToWideCharEx
+# define StringToWideChar StringMultiByteToWideChar
+# define StringCanonicalizeEx StringCanonicalizeExA
+# define StringCanonicalize StringCanonicalizeA
+#endif /* not UNICODE */
 
 
-/*  文字列を分割する
-    lpszString,文字列
-    cDelimiter,文字
-           RET,分割された文字列                                             */
-LPSTR * WINAPI
-StringSplitA (LPCSTR lpszString,
-              CHAR   cDelimiter);
+/******************************************************************************
+*                                                                             *
+******************************************************************************/
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+              RET,文字列                                                    */
+LPWSTR WINAPI
+StringJoinW (LPCWSTR lpszSeparator, ...);
+
+
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+              RET,文字列                                                    */
+LPSTR WINAPI
+StringJoinA (LPCSTR lpszSeparator, ...);
 
 
 /*  文字列を結合する
@@ -323,68 +449,178 @@ StringJoinVA (LPCSTR  lpszSeparator,
               LPCSTR *plpszArray);
 
 
-/*  文字列を結合する
-    lpszSeparator,区切り文字列
-              RET,文字列                                                    */
-LPWSTR WINAPI
-StringJoinW (LPCWSTR lpszSeparator, ...);
-
-
-/*  文字列を結合する
-    lpszSeparator,区切り文字列
-              RET,文字列                                                    */
-LPSTR WINAPI
-StringJoinA (LPCSTR lpszSeparator, ...);
+/*  分割された文字列の数を求める
+    plpszArray,分割された文字列
+           RET,分割された文字列の数                                         */
+int WINAPI
+StringLengthVW (LPCWSTR *plpszArray);
 
 
 /*  分割された文字列の数を求める
     plpszArray,分割された文字列
            RET,分割された文字列の数                                         */
 int WINAPI
-StringArrayLengthW (LPCWSTR *plpszArray);
+StringLengthVA (LPCSTR *plpszArray);
 
 
-/*  分割された文字列の数を求める
+/*  分割された文字列をコピーする
     plpszArray,分割された文字列
-           RET,分割された文字列の数                                         */
-int WINAPI
-StringArrayLengthA (LPCSTR *plpszArray);
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI
+StringCopyVW (LPCWSTR *plpszArray);
+
+
+/*  分割された文字列をコピーする
+    plpszArray,分割された文字列
+           RET,分割された文字列                                             */
+LPSTR * WINAPI
+StringCopyVA (LPCSTR *plpszArray);
+
+
+/*  分割された文字列のNULL文字列を削除する
+    plpszArray,分割された文字列
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI
+StringShrinkVW (LPCWSTR *plpszArray);
+
+
+/*  分割された文字列のNULL文字列を削除する
+    plpszArray,分割された文字列
+           RET,分割された文字列                                             */
+LPSTR * WINAPI
+StringShrinkVA (LPCSTR *plpszArray);
 
 
 #ifdef UNICODE
-# define wvasprintf wvasprintfW
-# define wasprintf wasprintfW
-# define StringDuplicateEx StringDuplicateExW
-# define StringDuplicate StringDuplicateW
-# define StringFromMultiByteEx StringMultiByteToWideCharEx
-# define StringFromMultiByte StringMultiByteToWideChar
-# define StringFromWideCharEx StringDuplicateExW
-# define StringFromWideChar StringDuplicateW
-# define StringToMultiByteEx StringWideCharToMultiByteEx
-# define StringToMultiByte StringWideCharToMultiByte
-# define StringToWideCharEx StringDuplicateExW
-# define StringToWideChar StringDuplicateW
-# define StringCanonicalizeEx StringCanonicalizeExW
-# define StringCanonicalize StringCanonicalizeW
-# define StringSplit StringSplitW
-# define StringArrayLength StringArrayLengthW
+# define StringJoin StringJoinW
+# define StringJoinV StringJoinVW
+# define StringLengthV StringLengthVW
+# define StringCopyV StringCopyVW
+# define StringShrinkV StringShrinkVW
 #else /* not UNICODE */
-# define wvasprintf wvasprintfA
-# define wasprintf wasprintfA
-# define StringDuplicateEx StringDuplicateExA
-# define StringDuplicate StringDuplicateA
-# define StringFromMultiByteEx StringDuplicateExA
-# define StringFromMultiByte StringDuplicateA
-# define StringFromWideCharEx StringWideCharToMultiByteEx
-# define StringFromWideChar StringWideCharToMultiByte
-# define StringToMultiByteEx StringDuplicateExA
-# define StringToMultiByte StringDuplicateA
-# define StringToWideCharEx StringMultiByteToWideCharEx
-# define StringToWideChar StringMultiByteToWideChar
-# define StringCanonicalizeEx StringCanonicalizeExA
-# define StringCanonicalize StringCanonicalizeA
-# define StringSplit StringSplitA
-# define StringArrayLength StringArrayLengthA
+# define StringJoin StringJoinA
+# define StringJoinV StringJoinVA
+# define StringLengthV StringLengthVA
+# define StringCopyV StringCopyVA
+# define StringShrinkV StringShrinkVA
+#endif /* not UNICODE */
+
+
+/******************************************************************************
+*                                                                             *
+******************************************************************************/
+typedef int (CALLBACK *StringSplitProcW_t) (LPCWSTR lpszString,
+                                            LPVOID  lpData);
+typedef int (CALLBACK *StringSplitProcA_t) (LPCSTR lpszString,
+                                            LPVOID lpData);
+
+
+/*  文字列を分割する
+         lpszString,文字列
+    StringSplitProc,コールバック関数
+             lpData,ユーザデータ
+                RET,分割された文字列                                        */
+LPWSTR * WINAPI
+StringSplitW (LPCWSTR            lpszString,
+              StringSplitProcW_t StringSplitProc,
+              LPVOID             lpData);
+
+
+/*  文字列を分割する
+         lpszString,文字列
+    StringSplitProc,コールバック関数
+             lpData,ユーザデータ
+                RET,分割された文字列                                        */
+LPSTR * WINAPI
+StringSplitA (LPCSTR             lpszString,
+              StringSplitProcA_t StringSplitProc,
+              LPVOID             lpData);
+
+
+/*  文字列を分割文字列の中の1文字で分割する
+    lpszString,文字列
+      lpszData,分割文字列
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI StringSplitDelimiterW   (LPCWSTR lpszString, LPCWSTR lpszData);
+LPSTR  * WINAPI StringSplitDelimiterA   (LPCSTR  lpszString, LPCSTR  lpszData);
+
+
+/*  文字列を分割文字列の中の1文字で分割する
+    lpszString,文字列
+      lpszData,分割文字列(大文字小文字を区別しない)
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI StringSplitDelimiterIW  (LPCWSTR lpszString, LPCWSTR lpszData);
+LPSTR  * WINAPI StringSplitDelimiterIA  (LPCSTR  lpszString, LPCSTR  lpszData);
+
+
+/*  文字列を分割文字列の中の1文字に一致する文字列で分割する
+    lpszString,文字列
+      lpszData,分割文字列
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI StringSplitDelimitersW  (LPCWSTR lpszString, LPCWSTR lpszData);
+LPSTR  * WINAPI StringSplitDelimitersA  (LPCSTR  lpszString, LPCSTR  lpszData);
+
+
+/*  文字列を分割文字列の中の1文字に一致する文字列で分割する
+    lpszString,文字列
+      lpszData,分割文字列(大文字小文字を区別しない)
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI StringSplitDelimitersIW (LPCWSTR lpszString, LPCWSTR lpszData);
+LPSTR  * WINAPI StringSplitDelimitersIA (LPCSTR  lpszString, LPCSTR  lpszData);
+
+
+/*  文字列を分割文字列で分割する
+    lpszString,文字列
+      lpszData,分割文字列
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI StringSplitSeparatorW   (LPCWSTR lpszString, LPCWSTR lpszData);
+LPSTR  * WINAPI StringSplitSeparatorA   (LPCSTR  lpszString, LPCSTR  lpszData);
+
+
+/*  文字列を分割文字列で分割する
+    lpszString,文字列
+      lpszData,分割文字列(大文字小文字を区別しない)
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI StringSplitSeparatorIW  (LPCWSTR lpszString, LPCWSTR lpszData);
+LPSTR  * WINAPI StringSplitSeparatorIA  (LPCSTR  lpszString, LPCSTR  lpszData);
+
+
+/*  文字列を分割文字列の中のどの文字にも一致しない文字列で分割する
+    lpszString,文字列
+      lpszData,分割文字列
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI StringSplitComplementW  (LPCWSTR lpszString, LPCWSTR lpszData);
+LPSTR  * WINAPI StringSplitComplementA  (LPCSTR  lpszString, LPCSTR  lpszData);
+
+
+/*  文字列を分割文字列の中のどの文字にも一致しない文字列で分割する
+    lpszString,文字列
+      lpszData,分割文字列(大文字小文字を区別しない)
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI StringSplitComplementIW (LPCWSTR lpszString, LPCWSTR lpszData);
+LPSTR  * WINAPI StringSplitComplementIA (LPCSTR  lpszString, LPCSTR  lpszData);
+
+
+#ifdef UNICODE
+# define StringSplit            StringSplitW
+# define StringSplitDelimiter   StringSplitDelimiterW
+# define StringSplitDelimiterI  StringSplitDelimiterIW
+# define StringSplitDelimiters  StringSplitDelimitersW
+# define StringSplitDelimitersI StringSplitDelimitersIW
+# define StringSplitSeparator   StringSplitSeparatorW
+# define StringSplitSeparatorI  StringSplitSeparatorIW
+# define StringSplitComplement  StringSplitComplementW
+# define StringSplitComplementI StringSplitComplementIW
+#else /* not UNICODE */
+# define StringSplit            StringSplitA
+# define StringSplitDelimiter   StringSplitDelimiterA
+# define StringSplitDelimiterI  StringSplitDelimiterIA
+# define StringSplitDelimiters  StringSplitDelimitersA
+# define StringSplitDelimitersI StringSplitDelimitersIA
+# define StringSplitSeparator   StringSplitSeparatorA
+# define StringSplitSeparatorI  StringSplitSeparatorIA
+# define StringSplitComplement  StringSplitComplementA
+# define StringSplitComplementI StringSplitComplementIA
 #endif /* not UNICODE */
 
 
@@ -625,11 +861,14 @@ ListEnum (LPLIST         lpStart,
 *                                                                             *
 ******************************************************************************/
 typedef struct _HASH *LPHASH;
-typedef BOOL (WINAPI *HashCompare_t)(LPVOID lpKey1, LPVOID lpKey2,
-                                                                LPVOID lpData);
-typedef VOID (WINAPI *HashFree_t)(LPVOID lpObject, LPVOID lpData);
-typedef BOOL (CALLBACK *HashEnumProc_t)(LPVOID lpKey, LPVOID lpValue,
-                                                                LPVOID lpData);
+typedef BOOL (WINAPI *HashCompare_t )(LPVOID lpKey1,
+                                      LPVOID lpKey2,
+                                      LPVOID lpData);
+typedef VOID (WINAPI *HashFree_t) (LPVOID lpObject,
+                                   LPVOID lpData);
+typedef BOOL (CALLBACK *HashEnumProc_t) (LPVOID lpKey,
+                                         LPVOID lpValue,
+                                         LPVOID lpData);
 
 
 /*  鍵を直接比較する
