@@ -1,6 +1,6 @@
 /*
     peimage
-    copyright (c) 1998-2017 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2018 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include <windows.h>
-#include "common.h"
+#include "wcommon/wcommon.h"
 
 
 #define PECHECK_HEADER 0
