@@ -1,6 +1,6 @@
 /*
     snap
-    copyright (c) 2000-2017 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
+    copyright (c) 2000-2018 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 */
 #include <windows.h>
 #include <tchar.h>
-#include "common.h"
+#include "wcommon/wcommon.h"
 #include "peimage.h"
 #include "snap.h"
 
